_F='key_hash'
_E='enterprise'
_D='timestamp'
_C='token'
_B='key'
_A=False
import base64,json,logging,os,traceback
from typing import Dict,Union
from localstack import config as localstack_config
from localstack.utils.http import get_proxies
from localstack.utils.http import safe_requests as requests
from localstack.utils.json import FileMappedDocument
from localstack.utils.strings import md5,str_insert,str_remove,to_bytes,to_str
from localstack.utils.time import now_utc
from localstack_ext import __version__,config
from localstack_ext.bootstrap import auth
from localstack_ext.bootstrap.auth import AuthClient,AuthToken
LOG=logging.getLogger(__name__)
ENV_PREPARED={}
MAX_KEY_CACHE_DURATION_SECS=60*60*24
ENV_LOCALSTACK_API_KEY='LOCALSTACK_API_KEY'
AuthCache=Union[FileMappedDocument,Dict]
class KeyActivationError(Exception):
	def get_user_friendly(A):return f"""
===============================================
API key activation failed! 🔑❌

{A}

Due to this error, LocalStack has quit. LocalStack pro features can only be used with a valid license.

- Please check that your API key is set up correctly and that you are using the correct key.
  You can find your API key in our webapp at https://app.localstack.cloud.
- If you want to continue using LocalStack without pro features you can set `ACTIVATE_PRO=0`.
"""
class CachedKeyError(KeyActivationError):0
class InvalidKeyError(KeyActivationError):
	def __init__(A,api_key,message_from_server):super().__init__(f"The API key you provided in the `{ENV_LOCALSTACK_API_KEY}` environment variable '{truncate_api_key(api_key)}' could not be activated against our licensing server. Server message: {message_from_server}.")
class InvalidDecryptionKeyError(KeyActivationError):0
class MissingCredentialsException(KeyActivationError):
	def __init__(A):super().__init__(f"We could not get the API Key from the environment variable `{ENV_LOCALSTACK_API_KEY}`")
def is_enterprise():return read_api_key(raise_if_missing=_A)==_E
def read_api_key(raise_if_missing=True):
	A=(os.environ.get(ENV_LOCALSTACK_API_KEY)or'').strip()
	if not A and raise_if_missing:raise MissingCredentialsException()
	return A
def truncate_api_key(api_key):A=api_key;return'"%s..."(%s)'%(A[:3],len(A))
def fetch_key():
	F='py.warnings';A=read_api_key()
	if A=='test':return b'test'
	elif A==_E:LOG.debug('Looking for a cached enterprise key, skipping online activation.');C=load_cached_key(api_key=A,check_timeout=_A);D=base64.b64decode(C);return D
	G=get_proxies()or None;H={'api_key':A,'version':__version__}
	try:
		logging.getLogger(F).setLevel(logging.ERROR);B=requests.post('%s/activate'%config.API_URL,json.dumps(H),verify=_A,proxies=G)
		if B.status_code>=400:
			E=B.content;I=B.headers.get('Content-Type')
			if B.status_code==403:J=json.loads(to_str(E))['message'];raise InvalidKeyError(A,J)
			raise KeyActivationError('Unexpected error while talking to the activation server (code %s): ctype "%s" - %s'%(B.status_code,I,E))
		C=json.loads(to_str(B.content))[_B];cache_key_locally(A,C)
	except InvalidKeyError:raise
	except Exception as K:
		if log_license_issues():A=str(api_key_configured()or'');LOG.warning('Error activating API key %s: %s %s'%(truncate_api_key(A),K,traceback.format_exc()));LOG.warning('Looking for cached key as fallback...')
		C=load_cached_key(A)
	finally:logging.getLogger(F).setLevel(logging.WARNING)
	D=base64.b64decode(C);return D
def get_key_cache():A=localstack_config.dirs.cache if not is_enterprise()else localstack_config.dirs.static_libs;return FileMappedDocument(os.path.join(A,'key.json'),mode=384)
def cache_key_locally(api_key,key_b64):
	A=key_b64;B=str(int(now_utc()));C=to_str(base64.b64decode(A))
	for D in range(len(B)):C=str_insert(C,D*2,B[D])
	A=to_str(base64.b64encode(to_bytes(C)));E=get_key_cache();E.update({_D:int(B),_F:md5(api_key),_B:A});E.save()
def load_cached_key(api_key,check_timeout=True):
	A=get_key_cache()
	if not A.get(_B):raise CachedKeyError('Could not find cached key')
	if A.get(_F)!=md5(api_key):raise CachedKeyError('Cached key was created for a different API key')
	E=now_utc()
	if check_timeout and E-A[_D]>MAX_KEY_CACHE_DURATION_SECS:raise CachedKeyError('Cached key expired')
	D=str(A[_D]);B=to_str(base64.b64decode(A[_B]))
	for C in range(len(D)):assert B[C]==D[C];B=str_remove(B,C)
	F=to_str(base64.b64encode(to_bytes(B)));return F
def log_license_issues():return api_key_configured()and localstack_config.is_env_not_false('LOG_LICENSE_ISSUES')
def api_key_configured():return config.is_api_key_configured()
def is_logged_in():return True if auth.get_auth_cache().get(_C)else _A
def get_auth_headers(auth_cache=None):
	B=auth_cache;B=B or auth.get_auth_cache();C=B.get(_C);A=C
	if isinstance(A,dict):
		H=AuthClient();D=AuthToken(C.get(_C),metadata=C);D=H.refresh_token(D);E=D.to_dict()
		if C!=E:C.update(E);B[_C]=C;B.save()
		A=D.token
	if A:
		I=B.get('provider')or'internal';F=f"{I} "
		if not A.startswith(F)and' 'not in A:A=f"{F}{A}"
		return{'authorization':A}
	G=read_api_key(raise_if_missing=_A)
	if G:return{'ls-api-key':G,'ls-version':__version__}
	raise Exception('Please log in first')